@@ -2,5 +2,6 @@ from microprediction.conventions import MicroConventions, new_key
 from microprediction.reader import MicroReader
 from microprediction.writer import MicroWriter
 from microprediction.crawler import MicroCrawler
+from microprediction.simplecrawler import SimpleCrawler, RegularCrawler
 from microprediction.supporter import donate
-
+from microprediction.looping import PandasLoop
