@@ -1,11 +1,32 @@
 # Provides an example of a self-navigating algorithm which you are under no obligation to use
 from microprediction.writer import MicroWriter
+from microprediction import new_key
 from microconventions import api_url
 import random, time, datetime
 from microprediction.samplers import exponential_bootstrap, approx_mode
 import pprint
 import numpy as np
 from statistics import median
+import os
+
+
+def new_key_if_none(write_key, use_environ=False, difficulty=10 ):
+    """ Create key on the fly but warn user about time this may take
+
+          :param   write_key str
+          :param   use_environ bool     Highly experimental ... set True at your own risk
+          :param   difficulty  int
+          :returns write_key
+    """
+    if use_environ:
+        write_key = write_key or os.environ.get('WRITE_KEY')
+    if write_key is None:
+        print('A write_key was not supplied, so burning one now of difficulty '+str(difficulty)+'. This will take a while, after which your crawler will be initialized. Save the write_key and next time supply a write_key to the constructor. ', flush=True)
+        write_key = new_key(difficulty=difficulty)
+        print('Your write_key is ' + write_key + '. Do not lose!! We recommend a quick visit to http://dev.microprediction.org/dashboard.html where you can punch it in')
+        if use_environ:
+            os.environ['WRITE_KEY'] = write_key
+    return write_key
 
 
 class MicroCrawler(MicroWriter):
@@ -109,8 +130,8 @@ class MicroCrawler(MicroWriter):
     def sample(self, lagged_values, lagged_times=None, name=None, delay=None, **ignored):
         """ An example of a sample method. This is where all the intelligence goes
 
-               :param lagged_times [ float ]    Vector with most recent listed first
-               :param lagged_times [ float ]    Vector of epoch times, most recent listed first
+               :param lagged_times [ float ]    List with most recent listed first
+               :param lagged_times [ float ]    List of epoch times, most recent listed first
                :param name          str         Name of stream
                :param delay         int         Prediction horizon in seconds
                **ignored                        For future compatability.
@@ -166,7 +187,7 @@ class MicroCrawler(MicroWriter):
     #############################################################################################
 
 
-    def __init__(self, write_key=None, base_url=None, verbose=False, quietude=50, stop_loss=10, min_budget=0., max_budget=10, min_lags = 25, max_lags=1000000, sponsor_min=12, **ignored):
+    def __init__(self, write_key=None, base_url=None, verbose=False, quietude=50, stop_loss=10, min_budget=0., max_budget=10, min_lags = 25, max_lags=1000000, sponsor_min=12, use_environ=False, difficulty=10, **ignored):
         """
             param: write_key  str    Valid write_key    See www.microprediction.org or www.muid.com for more details
             param: base_url   str    e.g  'http://api.microprediction.org'  or 'http://devapi.microprediction.org' for the brave. Defaults to what is at http://config.microprediction.org
@@ -178,13 +199,15 @@ class MicroCrawler(MicroWriter):
             param: min_lags   int
             param: max_lags   int    These set the short/long limits for the number of lags in a time series. Does your algo need or want a lot of data?
             param: sponsor_min int   Minimum write_key difficulty of the stream sponsor. E.g. set to 13 to only look at streams where people bothered to generate a 13-MUID.
-            param: **ignored         For future compatability
+            param: use_environ bool   Look in environ and store write_key
+            param: difficulty  int   Used to create key if none supplied, but otherwise ignored
+            param: **ignored        For future compatability
 
         """
 
         # Parameter management
-        super().__init__(base_url=base_url or api_url(), write_key=write_key, verbose=verbose )
-        assert self.key_difficulty(write_key) >= 7, "Invalid write_key for crawler. See www.muid.org to mine one. "
+        super().__init__(base_url=base_url or api_url(), write_key=new_key_if_none(write_key,use_environ=use_environ,difficulty=difficulty), verbose=verbose )
+        assert self.key_difficulty(self.write_key) >= 7, "Invalid write_key for crawler. Use new_key(difficulty=11) to create one. "
         assert not self.base_url[-1]=='/','Base url should not have trailing /'
         assert stop_loss>0,"Stop loss must be positive "
         self.quietude    = int(quietude)       # e.g. if set to 10, will only print 1/10th of the time
@@ -206,6 +229,7 @@ class MicroCrawler(MicroWriter):
         self.last_performance_check = time.time()
         self.last_new_horizon = time.time()
         self.next_prediction_time = dict()  # Indexed by horizon
+        self.last_withdrawal_reset = time.time()
 
         # State - other
         self.withdrawn  = list()           # List of horizons we have withdrawn from, or will never enter
@@ -310,6 +334,8 @@ class MicroCrawler(MicroWriter):
             if dt is None:
                 dt = self.DELAYS[-1]
             assert dt > 0
+            if dt<10:
+                dt = 10
 
             # Don't wait more than 6 hours
             if dt>(60*60*6):
@@ -323,7 +349,6 @@ class MicroCrawler(MicroWriter):
         else:
             expected_at = time.time() + self.DELAYS[-1]
             dt = self.DELAYS[-1]
-            dt = self.DELAYS[-1]
         return expected_at, dt
 
 
@@ -335,7 +360,7 @@ class MicroCrawler(MicroWriter):
 
         # First determine the time until the next data point, but if that is very soon we want the one after
         expected_at, dt = self.expected_time_of_next_value(lagged_times=lagged_times)
-        assert dt>=55
+
         while expected_at - time.time() < 20:
             expected_at = expected_at + dt
 
@@ -444,14 +469,13 @@ class MicroCrawler(MicroWriter):
 
 
     def run(self,timeout=None):
-        # TODO: Make this more modular
         """
             The crawler visits streams. It maintains a list of expected times at which new data points will arrive. At the annointed time(s), it
             submits predictions after the new data has arrived. It periodically looks for new horizons. It periodically withdraws from horizons where
             it is not faring too well (watch out for 'die' and 'coin_*' time series as they are simple but can lead to fast losses for some stock standard
             time series algorithms).
 
-            This is just  suggestion. You can create very different crawlers using the MicroWriter class directly, should you wish to.
+            This is just a suggestion. You can create very different crawlers using the MicroWriter class directly, should you wish to.
 
         """
         print(self.animal + " restarting at " + str(datetime.datetime.now()), flush=True)
@@ -477,13 +501,20 @@ class MicroCrawler(MicroWriter):
         catching_up = True
         while time.time()<self.end_time:
 
+            # Reset withdrawal record just in case some occasionally fail ... defense
+            overdue_for_withdrawal_reset = time.time()-self.last_withdrawal_reset>20*60
+            if overdue_for_withdrawal_reset:
+                self.withdrawn = []
+                self.last_withdrawal_reset = time.time()
+                print('Resetting withdrawal records', flush=True)
+
             # Withdraw if need be from losing propositions
-            overdue_for_performance_check = (time.time()-self.last_performance_check > 10*60) or catching_up
+            overdue_for_performance_check = (time.time()-self.last_performance_check > 5*60) or catching_up
             if overdue_for_performance_check:
                 print('Checking performance ',flush=True)
                 self.performance = self.get_performance()  # Expensive operation and may attract a small charge in the future
                 self.active = self.get_active()
-                self.withdraw_from_worst_active(stop_loss=self.stop_loss, performance=self.performance, active=self.active, num=10)
+                self.withdraw_from_worst_active(stop_loss=self.stop_loss, performance=self.performance, active=self.active, num=50)
                 self.last_performance_check = time.time()
 
             # Maybe we look for a new horizon to predict, but do this
@@ -519,20 +550,19 @@ class MicroCrawler(MicroWriter):
             if self.seconds_until_next>0:
                 time.sleep(self.seconds_until_next)
 
-            # Now it is game time.
-            num_upcoming_to_consider = 100 if catching_up else 10
+            # Make predictions in rapid succession until a gap opens
+            go_time = time.time()
+            num_upcoming_to_consider = 10 if not catching_up else 100
             for horizon, seconds_to_go in self.upcoming(num=num_upcoming_to_consider,relative=True):
-                catching_up = False
-                # Hang if there isn't much time
-                if seconds_to_go>0 and seconds_to_go<2:
-                    time.sleep(seconds_to_go)
-                    seconds_to_go = -0.01
-                # Go time. Run through the list.
-                if seconds_to_go<0:
+                seconds_since_game_time = time.time()-go_time
+                adjusted_seconds_to_go = seconds_to_go-seconds_since_game_time
+                if adjusted_seconds_to_go<2:
+                    if adjusted_seconds_to_go>0:
+                        time.sleep(adjusted_seconds_to_go)
                     name, delay  = self.split_horizon_name(horizon)
                     lagged_times = self.get_lagged_times(name=name)
                     data_arrived_recently = lagged_times and (abs(time.time()-lagged_times[0])<30)
-                    if seconds_to_go<-30 or data_arrived_recently:
+                    if adjusted_seconds_to_go<-30 or data_arrived_recently:
                         name, delay = self.split_horizon_name(horizon)
                         self.predict_and_submit(name=name, delay=delay, lagged_times=lagged_times )
                     else:
@@ -540,6 +570,7 @@ class MicroCrawler(MicroWriter):
 
             # Be nice and don't overwhelm system
             time.sleep(1.5)
+            catching_up = False
 
         self.retirement_callback()
         self.status_report()
@@ -548,6 +579,13 @@ class MicroCrawler(MicroWriter):
         pprint.pprint(self.recent_updates())
 
 
+
+
+
+
+
+
+
 if __name__=="__main__":
     # Just an example write key... you'll need to get your own write_key  (see MUID.org or Micoprediction.org)
     from microprediction.config_private import FLASHY_COYOTE
